/*
 * complexmpI.h
 *
 * Copyright (c) 2024, Jordi-Lluís Figueras, Alex Haro
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


#ifndef __COMPLEXMPI_H
#define __COMPLEXMPI_H

#define COMPLEXMPI_VERSION 1.0

#include "realmp.h"
#include "realmpI.h"

class complexI
{
public:

  realI re;
  realI im;

  complexI();
  complexI(const complexI& y);
  complexI(const realI& r, const realI& i);
  complexI(const realI& r, const real& i);
  complexI(const real& r, const real& i);
  complexI(const char *a, const char *b);
  ~complexI();

  complexI& operator = (const complexI& y);
  complexI& operator = (const realI& y);
  complexI& operator = (const real& y);
};

// other functions defined as inlines

complexI  operator - (const complexI& x);
complexI  conj(const complexI& x);
complexI  operator + (const complexI& x, const complexI& y);
complexI  operator + (const complexI& x, const realI& y);
complexI  operator + (const realI& x, const complexI& y);
complexI  operator - (const complexI& x, const complexI& y);
complexI  operator - (const complexI& x, const realI& y);
complexI  operator - (const realI& x, const complexI& y);
complexI  operator * (const complexI& x, const complexI& y);
complexI  operator * (const complexI& x, const realI& y);
complexI  operator * (const realI& x, const complexI& y);
complexI operator / (const complexI& x, const complexI& y);
complexI operator / (const realI& x, const complexI& y);
complexI operator / (const complexI& x, const realI& r);
int is_inside(const complexI& a, const complexI& b);
complexI sqr(const complexI& x);
complexI cos(const complexI& x);
complexI sin(const complexI& x);
complexI sqrt(const complexI& x);
complexI pow(const complexI& x, int n);
complexI pow(const complexI& x, const realI& a);
realI abs(const complexI& x);

inline complexI::complexI ()
{
  re = realI("0");
  im = realI("0");
}

inline complexI::complexI (const complexI& y)
{
  re = y.re;
  im = y.im;
}

inline complexI::complexI(const realI& r, const realI& i)
{
  re = r;
  im = i;
}

inline complexI::complexI(const realI& r, const real& i)
{
  re = r;
  im = i;
}

inline complexI::complexI(const real& r, const real& i)
{
  re = r;
  im = i;
}

complexI::complexI (const char *a, const char *b) 
{
  mpfi_init_set_str(re.x, a, 10);
  mpfi_init_set_str(im.x, b, 10);
}

complexI::~complexI()
{
}

inline complexI& complexI::operator = (const complexI& y) 
{ 
  re = y.re; 
  im = y.im; 
  return *this; 
} 

inline complexI& complexI::operator = (const realI& y) 
{ 
  re = y; 
  im = realI("0"); 
  return *this; 
} 

inline complexI& complexI::operator = (const real& y) 
{ 
  re = y; 
  im = realI("0"); 
  return *this; 
} 

#endif
