/*
 * tgridmp.cc
 *
 * Copyright (c) 2024, Jordi-Lluís Figueras
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include<math.h>
#include"test.h"
#include<iostream>
#include"realmp.h"
#include"realmp.cpp"
#include"complexmp.h"
#include"complexmp.cpp"
#include"gridmp.h"
#include"gridmp.cpp"

using namespace std;

real tol;

int test_creation(void)
{
  if(silent == FALSE)
    printf("Testing creation of grid:");
  
  grid K;
  int *nn;

  nn = (int*) malloc(2*sizeof(int));

  nn[0] = 16;
  nn[1] = 16;
 
  K = grid(2, nn);
 
  free(nn);

  return TRUE;
}

int test_derivative(void)
{
  if(silent == FALSE)
    printf("Testing derivative of grid:");
  
  grid K, DK;
  real theta0, theta1;
  int *nn;
  int *index;

  nn = new int[2];

  nn[0] = 16;
  nn[1] = 16;
 
  K = grid(2, nn);
  index = new int[2];
  initialize_index_grid(index, 2, nn);
  for(int i = 0; i < K.nelem; i++)
  {
    theta0 = pi2*index[0]/nn[0]; 
    theta1 = pi2*index[1]/nn[1];
    K.elem[i] = sin(theta0);
    next_index_grid(index, 2, nn);
  }
 
  fourier Kf;
  Kf = fourier(2, nn);
  Kf = fft_F(K);
  K = fft_B(Kf);
  DK = derivative(K, 0);

  if(abs(DK.elem[0]-val1) > tol)
  {
    delete [] nn;
    delete [] index;
    return FALSE;
  }

  delete [] nn;
  delete [] index;

  return TRUE;
}

int main(int argc, char *argv[])
{
  ARG(argc, argv);

  START("Test tgridmp.x");

  int bits;
 
  for(bits = 30; bits < 1000; bits = bits+50)
  {
    realmp_allocate_cache(bits);
    
    tol = val1;

    for(int i = 0; i < bits/4*3; i++)
    {
      tol = tol/val2;
    }
    if(silent == FALSE)
    {
      printf("Number of bits = %d\n", bits);
      printf("tol = 2^(-3/4*bits)\n");
      mpfr_printf("tol = %.20Re\n", tol.x);
      printf("\n");
    }

    TEST(test_creation());
    TEST(test_derivative());
  }

  realmp_free_cache();

  END();

  return total-num_succ;
}


