/*
 * trealmpI.cc
 *
 * Copyright (c) 2024, Jordi-Lluís Figueras
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include<math.h>
#include"test.h"
#include<iostream>
#include"realmp.h"
#include"realmp.cpp"
#include"realmpI.h"
#include"realmpI.cpp"

using namespace std;

real tol;

int test_is_inside(void)
{
  realI a, b, c, d;

  a = unionI(realI(val0), realI(val1));
  b = unionI(realI(val1)/(realI(val4)), realI(val1)/(realI(val2)));
  c = unionI(realI(val0), realI(val1));
  d = unionI(realI(-val1)/(realI(val4)), realI(val1)/(realI(val2)));
  if(silent == FALSE)
    printf("Testing is_inside:\n");
  if(verbose == TRUE)
  {
    mpfr_printf("\t a = %.20RDe , %.20RUe\n", a.left_real().x, a.right_real().x);
    mpfr_printf("\t b = %.20RDe , %.20RUe\n", b.left_real().x, b.right_real().x);
    printf("Testing if a is contained in b: (YES)\n");
    mpfr_printf("\t c = %.20RDe , %.20RUe\n", c.left_real().x, c.right_real().x);
    mpfr_printf("\t d = %.20RDe , %.20RUe\n", d.left_real().x, d.right_real().x);
    printf("Testing if c is contained in d: (NO)\n");
  }
  printf("\t\t\t");
  if(is_inside(a, b) && is_inside(c, d))
  {
    return TRUE;
  }else
  {
    return FALSE;
  }
  return FALSE;
}

int test_sin0(void)
{
  realI a, c;
  a = val0I;
  c = sin(a);

  if(silent == FALSE)
    printf("Testing |sin(0)-0| < tol:");
  if(abs(a-c).right_real() < tol)
  {
    return TRUE;
  }else
  {
    return FALSE;
  }
  return FALSE;
}

int test_sinpi(void)
{
  realI a, c;
  a = piI;
  c = sin(a);

  if(silent == FALSE)
    printf("Testing |sin(pi)-0| < tol:");
  if(abs(val0I-c).right_real() < tol)
  {
    return TRUE;
  }else
  {
    return FALSE;
  }
  return FALSE;
}

int test_sin2pi(void)
{
  realI a, c;
  a = pi2I;
  c = sin(a);

  if(silent == FALSE)
    printf("Testing |sin(2pi)-0| < tol:");
  if(abs(val0I-c).right_real() < tol)
  {
    return TRUE;
  }else
  {
    return FALSE;
  }
  return FALSE;
}

int test_sinpidiv2(void)
{
  realI a, c;
  a = pihalfI;
  c = sin(a);

  if(silent == FALSE)
    printf("Testing |sin(pi/2)-1| < tol:");
  if(abs(val1-c).right_real() < tol)
  {
    return TRUE;
  }else
  {
    return FALSE;
  }
  return FALSE;
}

int test_cos0(void)
{
  realI a, c;
  a = val0I;
  c = cos(a);

  if(silent == FALSE)
    printf("Testing |cos(0)-1| < tol:");
  if(abs(val1I-c).right_real() < tol)
  {
    return TRUE;
  }else
  {
    return FALSE;
  }
  return FALSE;
}

int test_sqrt4(void)
{
  realI a, c;
  a = val4I;
  c = val2I;

  if(silent == FALSE)
    printf("Testing |sqrt(4)-2| < tol:");
  if(abs(sqrt(a)-c).right_real() < tol)
  {
    return TRUE;
  }else
  {
    return FALSE;
  }
  return FALSE;
}

int test_Newton_sqrt2(int bits)
{
  realI a, res;
  a = val1I;
  res = sqrt(val2);

  if(silent == FALSE)
    printf("Testing Newton sqrt(2):\n");
  if(silent == FALSE)
    printf("\t Number of iterations = %d\n", (int)floor(log(bits)/log(2)));
  for(int i = 0; i < log(bits)/log(2); i++)
  {
    a = a-(sqr(a)-val2)/(val2*a);
  }
  if(verbose == TRUE)
  {
    mpfr_printf("\t Newton approximation = %.20Re\n", a.right_real().x);
    mpfr_printf("\t sqrt(2) = %.20Re\n", res.right_real().x);
    mpfr_printf("\t Difference = %.20Re\n", (a-res).right_real().x);
    mpfr_printf("\t tol = %.20Re\n", tol.x);
  }

  if(silent == FALSE)
    printf("\t\t\t");
  if(abs(sqrt(val2I)-a).right_real() < tol)
  {
    return TRUE;
  }else
  {
    return FALSE;
  }
  return FALSE;
}

int test_eval_polynomials(void)
{
  realI res;
  realI *coef;
  realI x;
  int N;
  
  if(silent == FALSE)
    printf("Testing polynomial evaluation 1+x+x^2:\n");
  if(silent == FALSE)
    printf("\t\t\t");

  N = 3;
  coef = new realI[N];
  for(int i = 0; i < N; i++)
  {
    coef[i] = val1I;
  }
  for(int j = 0; j < 10; j++)
  {
    x = realI(j)/val10I;
    res = eval_pol(coef, x, N);
    if(abs(res-(val1I+x+sqr(x))).right_real() > tol)
    {
      delete [] coef;
      return FALSE;
    }
  }
  delete [] coef;
  return TRUE;
}

int test_taylor_sin_cos(int bits)
{
  realI resc;
  realI ress;
  realI *coefc;
  realI *coefs;
  realI x;
  int N;
  
  if(silent == FALSE)
    printf("Testing Taylor sin(x) and cos(x):\n");

  N = bits;
  coefc = new realI[N];
  coefs = new realI[N];
  for(int i = 0; i < N; i++)
  {
    coefc[i] = val0I;
    coefs[i] = val0I;
  }
  coefc[0] = val1I;
  coefs[0] = val0I;
  for(int i = 1; i < N; i++)
  {
    //coefc[i] = val0I-coefs[i-1]/realI(i);
    coefc[i] = (-coefs[i-1])/realI(i);
    coefs[i] = coefc[i-1]/realI(i);
  }
  for(int j = 0; j < 10; j++)
  {
    x = realI(j)/val10I/val2I;
    resc = eval_pol(coefc, x, N);
    ress = eval_pol(coefs, x, N);
    if(verbose == TRUE)
    {
      printf("Testing cos(%d/20) and sin(%d/20)\n", j, j);
      mpfr_printf("\t taylor cos(%d/20) = %.20Re\n", j, resc.right_real().x);
      mpfr_printf("\t abs(taylor cos(%d/20)-cos(%d/20)) = %.20Re\n", j, j, abs(cos(x)-resc).right_real().x);
      mpfr_printf("\t taylor sin(%d/20) = %.20Re\n", j, ress.right_real().x);
      mpfr_printf("\t abs(taylor sin(%d/20)-sin(%d/20)) = %.20Re\n", j, j, abs(sin(x)-ress).right_real().x);
    }
    if(abs(resc-cos(x)).left_real() > tol || abs(ress-sin(x)).left_real() > tol)
    {
      printf("\t\t\t");
      delete [] coefc;
      delete [] coefs;
      return FALSE;
    }
  }
  if(silent == FALSE)
    printf("\t\t\t");
  delete [] coefc;
  delete [] coefs;
  return TRUE;

}

int test_digits(void)
{
  int digits;
  int bits, count;

  if(silent == FALSE)
    printf("Testing prec_digits_to_bits:\n");
  
  for(int  i = 20; i < 1000; i = i+50)
  {
    real eps;
    
    digits = i;
    bits = prec_digits_to_bits(digits);

    realmp_allocate_cache(bits);

    eps = val1;

    count = 0;
    do
    {
      count ++;
      eps = eps/val2;
    }while(val1+eps > val1);
    
    if(verbose == TRUE)
    {
      printf("Number of desired digits = %d\n", digits);
      mpfr_printf("Number of obtained digits = %.0Rf\n", floor(real(count)*log(val2)/log(val10)));
    }
    if(real(count) < real(digits)*log(val10)/log(val2))
    {
      printf("asf = %d\n", count);
      mpfr_printf("asdf = %.0Rf\n", real(digits)*log(val10)/log(val2));
      if(silent == FALSE)
        printf("\t\t\t");
      return FALSE;
    }
  }
  if(silent == FALSE)
    printf("\t\t\t");
  return TRUE;
}

int main(int argc, char *argv[])
{
  ARG(argc, argv);

  START("Test trealmp.x");

  int bits;
 
  for(bits = 30; bits < 1000; bits = bits+50)
  {
    realmpI_allocate_cache(bits);
    realmp_allocate_cache(bits);
    
    tol = val1;

    for(int i = 0; i < bits/4*3; i++)
    {
      tol = tol/val2;
    }
    if(silent == FALSE)
    {
      printf("Number of bits = %d\n", bits);
      printf("tol = 2^(-3/4*bits)\n");
      mpfr_printf("tol = %.20Re\n", tol.x);
      printf("\n");
    }

    TEST(test_is_inside());
    TEST(test_sin0());
    TEST(test_sinpi());
    TEST(test_sin2pi());
    TEST(test_sinpidiv2());
    TEST(test_cos0());
    TEST(test_sqrt4());
    TEST(test_Newton_sqrt2(bits));
    TEST(test_eval_polynomials());
    TEST(test_taylor_sin_cos(bits));
  }

//  TEST(test_digits());
  
  realmp_free_cache();

  END();

  return total-num_succ;
}


